// SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
// SPDX-FileCopyrightText: 2024 Harald Sitter <sitter@kde.org>

#pragma once

#include "categorymetadata.h"

namespace KNSCore
{

class CategoryMetadataPrivate
{
public:
    QString id;
    QString name;
    QString displayName;
};

} // namespace KNSCore
